#!/usr/bin/env python
# -*- coding: utf-8 -*-


import urllib2
import logging
import hashlib
import json


cache = {}

def getresource(url):
    '''Returns the resource at the url. '''

    #Check if already in cache, else add it
    key = hashlib.md5(url).hexdigest()
    if key not in cache:
        logging.info("Loading {}".format(url))
        response = urllib2.urlopen(url)
        cache[key] = json.load(response)

    return cache[key]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''

standings.py


'''

from collections import namedtuple


from teams import Team as Team
from resource import getresource as getresource


class Label(namedtuple('Label', "name, type")):
    __slots__ = () 
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('name', None),
            data.get('type', None)
        )




class Stats(object):    
    @classmethod    
    def from_list(cls, data):
        '''We get this as list of name/value pairs from the API'''
        
        obj = cls.__new__(cls)
        
        for stat in data:
            setattr(obj,stat['name'],stat['value'])
        
        return obj



class PositionStatus(namedtuple('PositionStatus', "name, type")):
    __slots__ = ()
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('name', None),
            data.get('type', None))


class PositionStatuses(list):
    @classmethod    
    def from_list(cls, data):
        '''We get this as list of name/value pairs from the API'''
        
        obj = cls.__new__(cls)
        
        for status in data:
            obj.append(PositionStatus.from_dict(status))
        
        return obj



class Standings(namedtuple('Standings', "team, stats, position_statuses")):
    __slots__ = ()
    @classmethod    
    def from_dict(cls, data):
        return cls(
            Team.from_dict(data.get('team', {})),
            Stats.from_list(data.get('stats',[])),
            PositionStatuses.from_list(data.get('positionStatuses',  []))
            )



class StandingsGroup(namedtuple('StandingsGroup', "labels, standings")):
    __slots__ = ()
    @classmethod    
    def from_dict(cls, data):

        labels = []
        for lbl in data.get('labels', []):
            labels.append(Label.from_dict(lbl))


        standings = []
        for sta in data.get('standings', []):
            standings.append(Standings.from_dict(sta))
            
        return cls(labels, standings)





class StandingsGroupsList(list):
    @classmethod
    def from_resource(cls, url):

        response = getresource(url)

        obj = cls.__new__(cls)             

        for group in response.get('groups', []):
            obj.append(StandingsGroup.from_dict(group))

        return obj


    def group(self,x=0):
        '''Returns the given group or None'''
        if len(self)>x:
            return self[x]
        else:
            return None

    def get_teamposition(self, team_id):
        '''Returns the position for a given team ID'''

        for group in self:
            for pos, teamstats in enumerate(group.standings,1):
                if teamstats.team.id == team_id:
                    return pos
        return None 


    def get_teamstats(self, team_id):
        '''Returns Stats for given team ID'''

        for group in self:
            for teamstats in group.standings:
                if teamstats.team.id == team_id:
                    return teamstats.stats
        return None                    


    def get_teams(self):
        '''Returns the list of teams in the standings'''
        teams = []
        for group in self:
            for teamstats in group.standings:
                teams.append(teamstats.team)
        return teams
    








#!/usr/bin/env python
# -*- coding: utf-8 -*-

from api import Api, EverysportException
import results

'''Teams from everysport.com
'''

from teams import Team 

MFF = Team(9375,u"Malmö FF", "", "")
HBG = Team(9373,u"Helingsborgs IF", "", "")
LSD = Team(1175,u"Leksand", "", "")
CHI = Team(28075,u"Chicago Blackhawks", "", "")



'''
	ID for everysport.com sports, as defined by 

	http://api.everysport.com/v1/sports


'''
AMERICAN_FOOTBALL = 18 #swedish: amerikansk fotboll
BADMINTON = 9
BANDY = 3
BASEBALL = 20
BASKET = 5
TABLE_TENNIS = 8
BOWLING = 1
WRESTLING = 16 #swedish: brottning
FOOTBALL = 10
HANDBALL = 7
FLOORBALL = 4
HOCKEY = 2
RUGBY = 17
SOFTBALL = 68
SPEEDWAY = 15
SQUASH = 22
TENNIS = 19
VOLLEYBALL = 11




'''
	Leagues
'''
#Football
ALLSVENSKAN = 57973
SUPERETTAN = 57974

#Hockey
SWISS_LEAGUE = 58882
SHL = 60243
NHL = 58878






#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''results.py

'''
from collections import namedtuple


class Result(namedtuple("Result", "team, home, round_, ga, gf, diff, pos, pos_change, against")):

    __slots__ = ()
    def _asjson(self):
        '''Returns a JSON serializable object'''
        return {
            'team' : self.team._asdict(),
            'against': self.against._asdict(),
            'home' : self.home,
            'round' : self.round_,
            'ga' : self.ga,
            'gf' : self.gf,
            'diff' : self.diff,
            'pos' : self.pos,
            'posChange' : self.pos_change
        }



def get_results_for_leagues(api_client, *league_ids):
    '''Generator for Results for given leagues.

    Arguments:
    league_ids - one or many league IDs
    '''

    for league_id in league_ids:
        for event in api_client.events(league_id).finished():

            #Get standings for this round
            standings = api_client.standings(event.league.id).round(event.round).fetchall()
         
            yield Result(
                event.home_team,
                True,
                event.round,
                event.visiting_team_score,
                event.home_team_score,
                event.home_team_score - event.visiting_team_score,
                standings.get_teamposition(event.home_team.id),
                0,
                event.visiting_team)


            #Add visiting team stats

            yield Result(
                event.visiting_team,
                False,
                event.round,
                event.home_team_score,
                event.visiting_team_score,
                event.visiting_team_score - event.home_team_score,
                standings.get_teamposition(event.visiting_team.id),
                0,
                event.home_team)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
A simple library that wraps the Everysport API. 

'''


import datetime
import logging


from events import EventsList as EventsList
from events import Event as Event
from standings import StandingsGroupsList as StandingsGroupsList
from results import get_results_for_leagues as get_results_for_leagues


import url_builder


class EverysportException(Exception):
    pass


class Api(object):
    '''Everysport API client'''


    def __init__(self, apikey):     
        '''Create an API client

        Arguments:
        apikey - See instructions at 
                https://github.com/menmo/everysport-api-documentation

        '''
        self.apikey = apikey


    def events(self, *league_ids):
        '''Returns a Query object for events from one or many leagues

        Arguments:
        league_ids - from everysport.com

        '''
        return EventsQuery(self, *league_ids)


    def standings(self, league_id):
        '''Returns a Query object for standings

        Arguments:
        league_id - from everysport.com

        '''
        return StandingsQuery(self, league_id)


    def get_results(self, *league_ids):
        '''Return results for given leagues

        Arguments:
        league_id - from everysport.com
        '''
        return get_results_for_leagues(self, *league_ids)
        

    def get_event(self, event_id):
        '''Returns an Event for a given id  

        Arguments:
        event_id - from everysport.com
        
        '''

        url = url_builder.get_event_url(event_id, apikey=self.apikey)

        try:
            event = Event.from_resource(url)    
        except Exception as e:
            m = u"Could not load event {} : {}".format(event_id, e.message)
            logging.warning(m)
            raise EverysportException(m)

        return event
            
 



class ApiQuery(object):
    '''Base class for all queries. '''
    def __init__(self, api_client):
        '''Initialized with the API and empty query '''
        self.api_client = api_client
        self.params = {}
        self.params['apikey'] = api_client.apikey


class StandingsQuery(ApiQuery):
    '''A Query for a league's standings (tables). In most leagues, just one. But e.g. NHL have many groups.
    '''    

    def __init__(self, api_client, league_id):
        super(StandingsQuery, self).__init__(api_client)
        self.league_id = league_id

    def total(self):
        '''Queries for Total standings '''
        self.params['type'] = 'total'
        return self

    def home(self):
        '''Queries for Home standings '''
        self.params['type'] = 'home'
        return self     

    def away(self):
        '''Queries for Away standings '''
        self.params['type'] = 'away'
        return self     

    def round(self, x):
        '''Queries for a specific round '''
        self.params['round'] = x
        return self 


    def fetchall(self):   
        '''Returns a StandingsGroupsList of all standings groups'''
        
        url = url_builder.get_standings_url(self.league_id, **self.params)

        return StandingsGroupsList.from_resource(url)





class EventsQuery(ApiQuery):
    '''A query for one or many events. 

    '''

    def __init__(self, api_client, *league_ids):
        super(EventsQuery, self).__init__(api_client)
        self.league_ids = league_ids


    def fromdate(self, d):
        '''Queries events from this date'''
        self.params['fromDate'] = d.strftime("%Y-%m-%d")
        return self

    
    def todate(self, d):
        '''Queries events to this date'''
        self.params['toDate'] = d.strftime("%Y-%m-%d")
        return self


    def today(self):
        '''Queries today's events'''
        today = datetime.date.today()
        self.params['toDate'] = self.params['fromDate'] = today.strftime('%Y-%m-%d')
        return self 


    def upcoming(self):
        '''Queries for all upcoming events'''
        self.params['status'] = "UPCOMING"
        return self

    def finished(self):
        '''Queries for all finished events'''
        self.params['status'] = "FINISHED"
        return self

    def sport(self, *sports):
        '''Queries events for one or many sports, by Sport ID'''
        self.params['sport'] = ",".join(map(str, sports))
        return self

    def teams(self, *teams):
        '''Queries events for one or many teams, by Team ID'''
        self.params['team'] = ",".join(map(str,teams))
        return self

    def round(self, *x):
        '''Queries for a specific round '''
        self.params['round'] = ",".join(map(str,x))
        return self 


    def fetchall(self):
        '''Returns a list of ALL events. This can be large, 500+ items for some leagues'''
        return list(self)


    def __iter__(self):
        '''Generator over events.'''

        done = False
        while not done:
            
            url = url_builder.get_events_url(*self.league_ids, **self.params)
            try:
                result = EventsList.from_resource(url)
            except:
                raise StopIteration
            
            done = result.count == 0 
            if not done:
                for ev in result:
                    yield ev
                self.params['offset'] = result.offset + result.count
            done = result.count < result.limit










#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
import logging

import everysport.url_builder

class TestUrls(unittest.TestCase):

    def test_event(self):
        url = everysport.url_builder.get_event_url(7126276, apikey="foo")
        logging.debug(url)
        self.assertTrue(url)


    def test_events(self):
        url = everysport.url_builder.get_events_url(57282, apikey="foo")
        logging.debug(url)
        self.assertTrue(url)


    def test_events_multipleleagues(self):
        url = everysport.url_builder.get_events_url(57282, 58883, apikey="foo")
        logging.debug(url)
        self.assertTrue(url)

    def test_events_withargs(self):
        url = everysport.url_builder.get_events_url(57282, apikey="foo", round=5)
        logging.debug(url)
        self.assertTrue(url)



    def test_standings(self):
        url = everysport.url_builder.get_standings_url(57282, apikey="foo")
        logging.debug(url)
        self.assertTrue(url)



if __name__ == '__main__': 
    logging.basicConfig(filename='url.log', level=logging.DEBUG, filemode="w")
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
import everysport
import logging
import os

'''Getting the Everysport APIKEY from the system environment. 

You need to set this with: 

    export EVERYSPORT_APIKEY={YOUR KEY}
'''
APIKEY = os.environ['EVERYSPORT_APIKEY'] 


class TestStandings(unittest.TestCase):

    def setUp(self):        
        self.api = everysport.Api(APIKEY)


    def test_standings(self):
        for standings in self.api.standings(everysport.ALLSVENSKAN).fetchall():
            self.assertTrue(len(standings.labels) >= 0)    
            self.assertTrue(len(standings) > 0)   


    def test_standings2(self):

        standings = self.api.standings(everysport.ALLSVENSKAN).fetchall()
        self.assertTrue(len(standings.group().labels) >= 0)
        self.assertTrue(len(standings.group().standings) >= 0)    



    def test_standings_swiss(self):
        for standings in self.api.standings(everysport.SWISS_LEAGUE).fetchall():
            self.assertTrue(len(standings.labels) >= 0)    
            self.assertTrue(len(standings) > 0)  


    def test_get_teamposition(self):
        standings = self.api.standings(everysport.ALLSVENSKAN).round(15).fetchall()
        pos = standings.get_teamposition(everysport.HBG.id)
        self.assertEqual(pos, 3)


    def test_get_teamposition2(self):
        standings = self.api.standings(everysport.ALLSVENSKAN).round(15).fetchall()
        pos = standings.get_teamposition(666666) #Invalid
        self.assertTrue(pos == None)

    

if __name__ == '__main__': 
    logging.basicConfig(filename='standings.log', 
                        level=logging.DEBUG, 
                        filemode="w") 
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
import everysport
import datetime
import logging
import os

'''Getting the Everysport APIKEY from the system environment. 

You need to set this with: 

    export EVERYSPORT_APIKEY={YOUR KEY}
'''
APIKEY = os.environ['EVERYSPORT_APIKEY'] 


class TestApi(unittest.TestCase):

    def setUp(self):        
        self.api = everysport.Api(APIKEY)


    def test_single_event(self):
        ev = self.api.get_event(2129667)
        self.assertEqual(ev.id, 2129667)

    
    def test_events_allsvenskan(self):
        events = self.api.events(everysport.ALLSVENSKAN).fetchall()
        self.assertTrue(len(events) == 240) #games in Allsvenskan  


    @unittest.skip("Too long")    
    def test_events_nhl(self):
        events = self.api.events(everysport.NHL).fetchall()
        self.assertTrue(len(events) == 720) #games in NHL  


    def test_events_allsvenskan_superettan(self):
        counter = 0
        for ev in self.api.events(
                        everysport.ALLSVENSKAN, 
                        everysport.SUPERETTAN):
            counter += 1
        self.assertTrue(counter == 480) #games in Allsvenskan  + Superettan

    

    def test_scores(self):
        for ev in self.api.events(everysport.ALLSVENSKAN).finished():    
            self.assertTrue(int(ev.home_team_score) >= 0)    
            self.assertTrue(int(ev.visiting_team_score) >= 0)



    def test_teams(self):
        for ev in self.api.events(everysport.ALLSVENSKAN):    
            self.assertTrue(len(ev.home_team.name) > 0 )    
            self.assertTrue(len(ev.visiting_team.name) > 0 )

    def test_round(self):
        for ev in self.api.events(everysport.ALLSVENSKAN).round(3):    
            self.assertEqual(ev.round, 3)  


    def test_rounds(self):
        for ev in self.api.events(everysport.ALLSVENSKAN).round(3,4,5,6):    
            self.assertTrue(ev.round >=3 or ev.round <= 6)  


    def test_team(self):
        for ev in self.api.events(everysport.SHL).teams(everysport.LSD.id):
            self.assertTrue(everysport.LSD.id in (ev.home_team.id, ev.visiting_team.id))  

    

    # Events status

    def test_finished(self):
        for ev in self.api.events(everysport.ALLSVENSKAN).finished():
            self.assertEqual(ev.status, "FINISHED")    


    def test_upcoming(self):
        for ev in self.api.events(everysport.ALLSVENSKAN).upcoming():
            self.assertEqual(ev.status, "UPCOMING")    


    # Events date

    def test_fromdate(self): 
        allsvenskan = self.api.events(everysport.ALLSVENSKAN)
        today = datetime.datetime.today()

        for ev in allsvenskan.fromdate(today):
            self.assertTrue(ev.start_date >= today)
            self.assertTrue(ev.time_zone == "CEST")





if __name__ == '__main__': 
    logging.basicConfig(filename='events.log', 
                        level=logging.DEBUG, 
                        filemode="w") 
    unittest.main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import namedtuple

class Team(namedtuple('Team', "id, name, short_name, abbreviation")):
    __slots__ = () #prevent creation of instance dict
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('id', None),
            data.get('name', None),
            data.get('short_name', None),
            data.get('abbreviation', None)
        )




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import everysport
import logging
import os

'''Getting the Everysport APIKEY from the system environment. 

You need to set this with: 

    export EVERYSPORT_APIKEY={YOUR KEY}
'''
APIKEY = os.environ['EVERYSPORT_APIKEY'] 


class TestApi(unittest.TestCase):

    def setUp(self):        
        self.api = everysport.Api(APIKEY)

    
    def test_teamslist(self):
        teams = self.api.get_teams(everysport.ALLSVENSKAN)
        self.assertTrue(len(teams) > 0)


if __name__ == '__main__': 
    logging.basicConfig(filename='teams.log', 
                        level=logging.DEBUG, 
                        filemode="w") 
    unittest.main()


from distutils.core import setup
setup(name='everysport',
      version='1.0.5',
      packages=['everysport'],
      description='A Python wrapper for the Everysport API',
      author='Peter Stark',
      author_email='peterstark72@gmail.com',
      url='https://github.com/peterstark72/everysport',
      classifiers=[
      	  'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: Free for non-commercial use',
          'Natural Language :: English',
          'Natural Language :: Swedish',
          'Operating System :: MacOS :: MacOS X',
          'Programming Language :: Python :: 2.7',
          'Topic :: Utilities'
          ],
      )


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''events.py


'''

import datetime
from collections import namedtuple


from teams import Team as Team
from resource import getresource as getresource


class Sport(namedtuple('Sport', "id, name")):
    __slots__ = () 
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('id', None),
            data.get('name', None)
        )


class Arena(namedtuple('Arena', "id, name")):
    __slots__ = () 
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('id', None),
            data.get('name', None)
        )


class Credit(namedtuple('Credit', "message, link, logoUrl")):
    __slots__ = () 
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('message', None),
            data.get('link', None),
            data.get('logoUrl', None)
        )



class League(namedtuple('League', "id, name, sport, team_class")):
    __slots__ = () 
    @classmethod
    def from_dict(cls, data):
        return cls(
            data.get('id', None),
            data.get('name', None),
            Sport.from_dict(data.get('sport', {})),
            data.get('team_class', None),
        )


class Facts(namedtuple('Facts', "arena, spectators, referees, shots")):
    __slots__ = ()
    @classmethod
    def from_dict(cls, data):
        return cls(
            Arena.from_dict(data.get('arena', {})),
            data.get('spectators', None),
            data.get('referees', []), #list of names
            data.get('shots', None),
        )



class Event(namedtuple('Event', "id, start_date, time_zone, round, status, home_team, visiting_team, home_team_score, visiting_team_score, finished_time_status, league,facts")):
    __slots__ = ()

    STATUS_FINISHED = "FINISHED"

    @classmethod    
    def from_dict(cls, data):
        try:
            d = data['startDate'][0:16] #Strip timezone
            start_date = datetime.datetime.strptime(d, "%Y-%m-%dT%H:%M")
        except:
            start_date = None

        return cls(
                data.get('id', None),
                start_date,
                "CEST", #All events today are in CEST timezone
                data.get('round',None),
                data.get('status',None),
                Team.from_dict(data.get('homeTeam',{})),
                Team.from_dict(data.get('visitingTeam',{})),
                data.get('homeTeamScore',None),
                data.get('visitingTeamScore',None),
                data.get('finishedTimeStatus',None),
                League.from_dict(data.get('league',{})),
                Facts.from_dict(data.get('facts', {}))
            )


    @classmethod    
    def from_resource(cls, url):

        data = getresource(url)     

        return cls.from_dict(data.get('event', {}))


    def is_finished(self):
        return self.status == Event.STATUS_FINISHED






class EventsList(list):
    '''A list of events returned from the API'''
    @classmethod
    def from_resource(cls, url):

        data = getresource(url)

        obj = cls.__new__(cls)

        for ev in data.get('events',[]):
            obj.append(Event.from_dict(ev))

        obj.credit = Credit.from_dict(data.get('credit', {}))
        
        obj.offset = data['metadata']['offset']
        obj.limit = data['metadata']['limit']
        obj.count = data['metadata']['count']

        return obj 



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
'''

import urllib
import logging

BASE_API_URL = "http://api.everysport.com/v1/{}"


def get_standings_url(league_id, **params):
    '''Builds URL for standings resource

    Arguments:
    league_id - from everysport.com

    '''
    url = BASE_API_URL.format('leagues/' + str(league_id) + '/standings')
    if len(params) > 0:
        return url + "?" + urllib.urlencode(params)
    return url


def get_events_url(*league_ids, **params):
    args = params
    url = BASE_API_URL.format('events')
    args['league'] = ",".join(map(str,league_ids))    
    return url + "?" + urllib.urlencode(args)    


def get_event_url(event_id, **params):
    url = BASE_API_URL.format('events/' + str(event_id)) + "?" + urllib.urlencode(params)
    logging.debug("Building {}".format(url))
    return url
    





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import everysport
import logging
import os


#Getting the Everysport APIKEY from the system environment. 
APIKEY = os.environ['EVERYSPORT_APIKEY'] 


class TestResults(unittest.TestCase):

    def setUp(self):        
        self.api = everysport.Api(APIKEY)  


    def test_results_allsvenskan(self):
        results = self.api.get_results(everysport.ALLSVENSKAN)
        self.assertTrue(len(list(results)) > 0)

    @unittest.skip("Takes time")        
    def test_results_nhl(self):
        results = self.api.get_results(everysport.NHL)
        self.assertTrue(len(list(results)) > 0)



if __name__ == '__main__': 
    logging.basicConfig(filename='results.log',
                            level=logging.DEBUG, 
                            filemode="w")
    unittest.main()
